@@ -2,8 +2,10 @@
 from protean.adapters.broker import Brokers
 from protean.adapters.broker.inline import InlineBroker
 from protean.adapters.cache import Caches
+from protean.adapters.cache.memory import MemoryCache
 from protean.adapters.email import EmailProviders
 from protean.adapters.email.dummy import DummyEmailProvider
+from protean.adapters.event_store.memory import MemoryEventStore
 from protean.adapters.repository import Providers
 from protean.adapters.repository.memory import MemoryModel, MemoryProvider
 
@@ -13,7 +15,9 @@ __all__ = (
     "DummyEmailProvider",
     "EmailProviders",
     "InlineBroker",
+    "MemoryCache",
     "MemoryModel",
     "MemoryProvider",
+    "MemoryEventStore",
     "Providers",
 )
