@@ -1,9 +1,10 @@
 """A Dummy Email Provider class"""
+
 import logging
 
 from protean.core.email import BaseEmailProvider
 
-logger = logging.getLogger("protean.adapters.email")
+logger = logging.getLogger(__name__)
 
 
 class DummyEmailProvider(BaseEmailProvider):
@@ -14,7 +15,7 @@ class DummyEmailProvider(BaseEmailProvider):
     def __init__(self, name, domain, conn_info):
         super().__init__(name, domain, conn_info)
 
-    def send_email(self, message, dynamic_template=False):
+    def send_email(self, message):
         """Output message into log"""
 
         logger.debug("Email message dispatched: %s" % message)
