@@ -1,12 +1,15 @@
-""" Define the send-grid email provider """
+"""Define the send-grid email provider"""
+
 import logging
 
+from python_http_client.exceptions import HTTPError
 from sendgrid import SendGridAPIClient
 from sendgrid.helpers.mail import Mail, TemplateId
 
 from protean.core.email import BaseEmailProvider
+from protean.exceptions import SendError
 
-logger = logging.getLogger("protean.email.sendgrid")
+logger = logging.getLogger(__name__)
 
 
 class SendgridEmailProvider(BaseEmailProvider):
@@ -16,8 +19,8 @@ class SendgridEmailProvider(BaseEmailProvider):
         super().__init__(*args, **kwargs)
         self.sg_client = SendGridAPIClient(self.conn_info["API_KEY"])
 
-    def send_email(self, message, dynamic_template=False):
-        """ Send messages via the sendgrid api"""
+    def send_email(self, message):
+        """Send messages via the sendgrid api"""
 
         email = Mail(
             from_email=message.from_email or self.conn_info["DEFAULT_FROM_EMAIL"],
@@ -31,11 +34,16 @@ class SendgridEmailProvider(BaseEmailProvider):
 
             if response.status_code != 202:
                 logger.error(
-                    f"Error encountered while sending Email: {response.status_code}"
+                    f"Failure: ({response.status_code}) - "
+                    f"{response.reason} - {response.body}"
                 )
 
             logger.debug("Email pushed to SendGrid successfully.")
+        except HTTPError as e:
+            logger.error(f"{e}: {e.to_dict}")
+            raise SendError(f"Exception: HTTPError - Failed to send email - {str(e)}")
         except Exception as e:
-            logger.error(f"Error encountered while sending Email: {e}")
+            logger.error(f"Exception: Error while sending email: {e}")
+            raise SendError(f"Exception: Failed to send email - {str(e)}")
 
         return True
