@@ -1,8 +1,7 @@
-"""Implementation of a dictionary based repository """
+"""Implementation of a dictionary based repository"""
 
 import copy
 import json
-
 from collections import defaultdict
 from datetime import date, datetime
 from itertools import count
@@ -11,43 +10,33 @@ from threading import Lock
 from typing import Any
 from uuid import UUID
 
-from protean.core.model import BaseModel
-from protean.exceptions import ObjectNotFoundError, ValidationError
-from protean.globals import current_uow
-from protean.port.dao import BaseDAO, BaseLookup, ResultSet
+from protean.core.database_model import BaseDatabaseModel
+from protean.core.queryset import ResultSet
+from protean.exceptions import DatabaseError, ObjectNotFoundError
+from protean.fields.basic import Auto
+from protean.port.dao import BaseDAO, BaseLookup
 from protean.port.provider import BaseProvider
-from protean.utils import Database
+from protean.utils.container import Options
+from protean.utils.globals import current_uow
 from protean.utils.query import Q
-
-# Global in-memory store of dict data. Keyed by name, to provide
-# multiple named local memory caches.
-_databases = defaultdict(dict)
-_locks = defaultdict(Lock)
-_counters = defaultdict(count)
+from protean.utils.reflection import attributes, fields, id_field
 
 
-def derive_schema_name(model_cls):
-    if hasattr(model_cls.meta_, "schema_name"):
-        return model_cls.meta_.schema_name
-    else:
-        return model_cls.meta_.entity_cls.meta_.schema_name
-
-
-class MemoryModel(BaseModel):
+class MemoryModel(BaseDatabaseModel):
     """A model for the dictionary repository"""
 
     @classmethod
     def from_entity(cls, entity) -> "MemoryModel":
-        """Convert the entity to a dictionary record """
+        """Convert the entity to a dictionary record"""
         dict_obj = {}
-        for attribute_name in entity.meta_.attributes:
+        for attribute_name in attributes(entity):
             dict_obj[attribute_name] = getattr(entity, attribute_name)
         return dict_obj
 
     @classmethod
     def to_entity(cls, item: "MemoryModel"):
-        """Convert the dictionary record to an entity """
-        return cls.meta_.entity_cls(item, raise_errors=False)
+        """Convert the dictionary record to an entity"""
+        return cls.meta_.part_of(**item)
 
 
 class MemorySession:
@@ -61,29 +50,16 @@ class MemorySession:
             self._db = current_uow._sessions[self._provider.name]._db
         else:
             self._db = {
-                "data": copy.deepcopy(_databases),
-                "lock": _locks.setdefault(self._provider.name, Lock()),
-                "counters": _counters,
+                "data": copy.deepcopy(self._provider._databases),
+                "lock": self._provider._locks.setdefault(self._provider.name, Lock()),
+                "counters": self._provider._counters,
             }
 
-    def add(self, element):
-        if element.state_.is_persisted:
-            dao = self._provider.get_dao(element.__class__)
-            dao.update(element, element.to_dict())
-        else:
-            dao = self._provider.get_dao(element.__class__)
-            dao.create(element.to_dict())
-
-    def delete(self, element):
-        dao = self._provider.get_dao(element.__class__)
-        dao.delete(element)
-
     def commit(self):
         if current_uow and self._provider.name in current_uow._sessions:
             current_uow._sessions[self._provider.name]._db["data"] = self._db["data"]
         else:
-            global _databases
-            _databases = self._db["data"]
+            self._provider._databases = self._db["data"]
 
     def rollback(self):
         pass
@@ -95,15 +71,21 @@ class MemorySession:
 class MemoryProvider(BaseProvider):
     """Provider class for Dict Repositories"""
 
+    __database__ = "memory"
+
     def __init__(self, name, domain, conn_info: dict):
         """Initialize Provider with Connection/Adapter details"""
 
-        # In case of `MemoryProvider`, the `DATABASE` value will always be `MEMORY`.
-        conn_info["DATABASE"] = Database.MEMORY.value
+        # In case of `MemoryProvider`, the `database` value will always be `memory`.
         super().__init__(name, domain, conn_info)
 
+        # Global in-memory store of dict data.
+        self._databases = defaultdict(dict)
+        self._locks = defaultdict(Lock)
+        self._counters = defaultdict(count)
+
         # A temporary cache of already constructed model classes
-        self._model_classes = {}
+        self._database_model_classes = {}
 
     def get_session(self):
         """Return a session object
@@ -115,82 +97,99 @@ class MemoryProvider(BaseProvider):
         return MemorySession(self)
 
     def get_connection(self, session_cls=None):
-        """Return the dictionary database object """
+        """Return the dictionary database object"""
         return MemorySession(self, new_connection=True)
 
+    def is_alive(self) -> bool:
+        """Check if the connection is alive"""
+        return True
+
     def _data_reset(self):
         """Reset data"""
-        global _databases, _locks, _counters
-        _databases = defaultdict(dict)
-        _locks = defaultdict(Lock)
-        _counters = defaultdict(count)
+        self._databases = defaultdict(dict)
+        self._locks = defaultdict(Lock)
+        self._counters = defaultdict(count)
 
         # Discard any active Unit of Work
         if current_uow and current_uow.in_progress:
             current_uow.rollback()
 
-    def decorate_model_class(self, entity_cls, model_cls):
-        schema_name = derive_schema_name(model_cls)
+    def close(self):
+        """Close the provider and clean up resources.
+
+        For MemoryProvider, this is a no-op since there are no persistent
+        connections or external resources to clean up.
+        """
+        pass
+
+    def decorate_database_model_class(self, entity_cls, database_model_cls):
+        schema_name = database_model_cls.derive_schema_name()
 
         # Return the model class if it was already seen/decorated
-        if schema_name in self._model_classes:
-            return self._model_classes[schema_name]
+        if schema_name in self._database_model_classes:
+            return self._database_model_classes[schema_name]
 
-        # If `model_cls` is already subclassed from MemoryModel,
+        # If `database_model_cls` is already subclassed from MemoryModel,
         #   this method call is a no-op
-        if issubclass(model_cls, MemoryModel):
-            return model_cls
+        if issubclass(database_model_cls, MemoryModel):
+            return database_model_cls
         else:
             custom_attrs = {
                 key: value
-                for (key, value) in vars(model_cls).items()
+                for (key, value) in vars(database_model_cls).items()
                 if key not in ["Meta", "__module__", "__doc__", "__weakref__"]
             }
 
-            from protean.core.model import ModelMeta
-
-            meta_ = ModelMeta()
-            meta_.entity_cls = entity_cls
+            meta_ = Options()
+            meta_.part_of = entity_cls
 
             custom_attrs.update({"meta_": meta_})
             # FIXME Ensure the custom model attributes are constructed properly
-            decorated_model_cls = type(
-                model_cls.__name__, (MemoryModel, model_cls), custom_attrs
+            decorated_database_database_model_cls = type(
+                database_model_cls.__name__,
+                (MemoryModel, database_model_cls),
+                custom_attrs,
             )
 
             # Memoize the constructed model class
-            self._model_classes[schema_name] = decorated_model_cls
+            self._database_model_classes[schema_name] = (
+                decorated_database_database_model_cls
+            )
 
-            return decorated_model_cls
+            return decorated_database_database_model_cls
 
-    def construct_model_class(self, entity_cls):
+    def construct_database_model_class(self, entity_cls):
         """Return associated, fully-baked Model class"""
-        model_cls = None
+        database_model_cls = None
 
         # Return the model class if it was already seen/decorated
-        if entity_cls.meta_.schema_name in self._model_classes:
-            model_cls = self._model_classes[entity_cls.meta_.schema_name]
+        if entity_cls.meta_.schema_name in self._database_model_classes:
+            database_model_cls = self._database_model_classes[
+                entity_cls.meta_.schema_name
+            ]
         else:
-            from protean.core.model import ModelMeta
-
-            meta_ = ModelMeta()
-            meta_.entity_cls = entity_cls
+            meta_ = Options()
+            meta_.part_of = entity_cls
 
             attrs = {
                 "meta_": meta_,
             }
             # FIXME Ensure the custom model attributes are constructed properly
-            model_cls = type(entity_cls.__name__ + "Model", (MemoryModel,), attrs)
+            database_model_cls = type(
+                entity_cls.__name__ + "Model", (MemoryModel,), attrs
+            )
 
             # Memoize the constructed model class
-            self._model_classes[entity_cls.meta_.schema_name] = model_cls
+            self._database_model_classes[entity_cls.meta_.schema_name] = (
+                database_model_cls
+            )
 
         # Set Entity Class as a class level attribute for the Model, to be able to reference later.
-        return model_cls
+        return database_model_cls
 
-    def get_dao(self, entity_cls, model_cls):
+    def get_dao(self, entity_cls, database_model_cls):
         """Return a DAO object configured with a live connection"""
-        return DictDAO(self.domain, self, entity_cls, model_cls)
+        return DictDAO(self.domain, self, entity_cls, database_model_cls)
 
     def _evaluate_lookup(self, key, value, negated, db):
         """Extract values from DB that match the given criteria"""
@@ -214,11 +213,17 @@ class MemoryProvider(BaseProvider):
 
         return results
 
-    def raw(self, query: Any, data: Any = None):
-        """Run raw queries on the database
+    def raw(self, query: Any, data: Any = None) -> list:
+        """Run raw queries on the memory database.
 
         As an example of running ``raw`` queries on a Dict repository, we will run the query
         on all possible schemas, and return all results.
+
+        For this stand-in repository, the query string is a json string that contains kwargs
+        criteria with straight-forward equality checks. Individual criteria are always AND-ed
+        and the result is always a subset of the full repository.
+
+        We will ignore the `data` parameter for this kind of repository.
         """
         assert isinstance(query, str)
 
@@ -246,9 +251,15 @@ class MemoryProvider(BaseProvider):
 
         return items
 
+    def _create_database_artifacts(self):
+        """Dummy placeholder. Nothing to do."""
+
+    def _drop_database_artifacts(self):
+        """Dummy placeholder. Nothing to do."""
+
 
 class DictDAO(BaseDAO):
-    """A repository for storing data in a dictionary """
+    """A repository for storing data in a dictionary"""
 
     def __repr__(self) -> str:
         return f"DictDAO <{self.entity_cls.__name__}>"
@@ -257,15 +268,16 @@ class DictDAO(BaseDAO):
         """Set the values of the auto field using counter"""
         conn = self._get_session()
 
-        for field_name in self.entity_cls.meta_.auto_fields:
-            counter_key = f"{self.schema_name}_{field_name}"
-            if not (field_name in model_obj and model_obj[field_name] is not None):
-                # Increment the counter and it should start from 1
-                counter = next(conn._db["counters"][counter_key])
-                if not counter:
+        for field_name, field_obj in fields(self.entity_cls).items():
+            if type(field_obj) is Auto and field_obj.increment:
+                counter_key = f"{self.schema_name}_{field_name}"
+                if not (field_name in model_obj and model_obj[field_name] is not None):
+                    # Increment the counter and it should start from 1
                     counter = next(conn._db["counters"][counter_key])
+                    if not counter:
+                        counter = next(conn._db["counters"][counter_key])
 
-                model_obj[field_name] = counter
+                    model_obj[field_name] = counter
 
         return model_obj
 
@@ -277,22 +289,23 @@ class DictDAO(BaseDAO):
         model_obj = self._set_auto_fields(model_obj)
 
         # Add the entity to the repository
-        identifier = model_obj[self.entity_cls.meta_.id_field.field_name]
+        identifier = model_obj[id_field(self.entity_cls).field_name]
         with conn._db["lock"]:
-            # Check if object is present
-            if identifier in conn._db["data"][self.schema_name]:
-                raise ValidationError(
-                    {
-                        "_entity": f"`{self.__class__.__name__}` object with identifier {identifier} "
-                        f"is already present."
-                    }
-                )
-
             conn._db["data"][self.schema_name][identifier] = model_obj
 
         if not current_uow:
-            conn.commit()
-            conn.close()
+            try:
+                conn.commit()
+            except Exception as exc:
+                # For memory provider, most errors would be during commit simulation
+                # This could include validation errors, constraint violations, etc.
+                conn.rollback()
+                raise DatabaseError(
+                    f"Database error during creation: {str(exc)}",
+                    original_exception=exc,
+                )
+            finally:
+                conn.close()
 
         return model_obj
 
@@ -350,42 +363,61 @@ class DictDAO(BaseDAO):
             if o_key.startswith("-"):
                 reverse = True
                 o_key = o_key[1:]
-            items = sorted(items, key=itemgetter(o_key), reverse=reverse)
+
+            null_items = [item for item in items if item.get(o_key) is None]
+            non_null_items = [item for item in items if item.get(o_key) is not None]
+            sorted_items = sorted(
+                non_null_items, key=itemgetter(o_key), reverse=reverse
+            )
+
+            # null values sort as if larger than any non-null value
+            # So in reverse (DESC) order, null values come first
+            # in ASC order, null values come last
+            if reverse:
+                items = null_items + sorted_items
+            else:
+                items = sorted_items + null_items
 
         result = ResultSet(
             offset=offset,
             limit=limit,
             total=len(items),
-            items=items[offset : offset + limit],
+            # When limit is not set, we return all items
+            items=items[offset : offset + limit] if limit else items,
         )
 
         return result
 
     def _update(self, model_obj):
-        """Update the entity record in the dictionary """
+        """Update the entity record in the dictionary"""
         conn = self._get_session()
 
-        identifier = model_obj[self.entity_cls.meta_.id_field.field_name]
+        identifier = model_obj[id_field(self.entity_cls).field_name]
         with conn._db["lock"]:
             # Check if object is present
             if identifier not in conn._db["data"][self.schema_name]:
                 raise ObjectNotFoundError(
-                    {
-                        "_entity": f"`{self.__class__.__name__}` object with identifier {identifier} "
-                        f"does not exist."
-                    }
+                    f"`{self.__class__.__name__}` object with identifier {identifier} "
+                    f"does not exist."
                 )
 
             conn._db["data"][self.schema_name][identifier] = model_obj
 
         if not current_uow:
-            conn.commit()
-            conn.close()
+            try:
+                conn.commit()
+            except Exception as exc:
+                conn.rollback()
+                raise DatabaseError(
+                    f"Database error during update: {str(exc)}", original_exception=exc
+                )
+            finally:
+                conn.close()
 
         return model_obj
 
     def _update_all(self, criteria: Q, *args, **kwargs):
-        """Update all objects satisfying the criteria """
+        """Update all objects satisfying the criteria"""
         conn = self._get_session()
 
         items = self._filter_items(criteria, conn._db["data"][self.schema_name])
@@ -400,31 +432,45 @@ class DictDAO(BaseDAO):
             update_count += 1
 
         if not current_uow:
-            conn.commit()
-            conn.close()
+            try:
+                conn.commit()
+            except Exception as exc:
+                conn.rollback()
+                raise DatabaseError(
+                    f"Database error during update_all: {str(exc)}",
+                    original_exception=exc,
+                )
+            finally:
+                conn.close()
 
         return update_count
 
     def _delete(self, model_obj):
-        """Delete the entity record in the dictionary """
+        """Delete the entity record in the dictionary"""
         conn = self._get_session()
 
-        identifier = model_obj[self.entity_cls.meta_.id_field.field_name]
+        identifier = model_obj[id_field(self.entity_cls).field_name]
         with conn._db["lock"]:
             # Check if object is present
             if identifier not in conn._db["data"][self.schema_name]:
                 raise ObjectNotFoundError(
-                    {
-                        "_entity": f"`{self.entity_cls.__name__}` object with identifier {identifier} "
-                        f"does not exist."
-                    }
+                    f"`{self.entity_cls.__name__}` object with identifier {identifier} "
+                    f"does not exist."
                 )
 
             del conn._db["data"][self.schema_name][identifier]
 
         if not current_uow:
-            conn.commit()
-            conn.close()
+            try:
+                conn.commit()
+            except Exception as exc:
+                conn.rollback()
+                raise DatabaseError(
+                    f"Database error during deletion: {str(exc)}",
+                    original_exception=exc,
+                )
+            finally:
+                conn.close()
 
         return model_obj
 
@@ -447,12 +493,20 @@ class DictDAO(BaseDAO):
                     del conn._db["data"][self.schema_name]
 
         if not current_uow:
-            conn.commit()
-            conn.close()
+            try:
+                conn.commit()
+            except Exception as exc:
+                conn.rollback()
+                raise DatabaseError(
+                    f"Database error during delete_all: {str(exc)}",
+                    original_exception=exc,
+                )
+            finally:
+                conn.close()
 
         return len(items)
 
-    def _raw(self, query: Any, data: Any = None):
+    def _raw(self, query: Any, data: Any = None) -> ResultSet:
         """Run raw query on Repository.
 
         For this stand-in repository, the query string is a json string that contains kwargs
@@ -461,35 +515,12 @@ class DictDAO(BaseDAO):
 
         We will ignore the `data` parameter for this kind of repository.
         """
-        # Ensure that we are dealing with a string, for this repository
-        assert isinstance(query, str)
+        items = self.provider.raw(query, data)
+        return ResultSet(offset=1, limit=len(items), total=len(items), items=items)
 
-        conn = self._get_session()
-        input_db = conn._db["data"][self.schema_name]
-        result = None
-
-        try:
-            # Ensures that the string contains double quotes around keys and values
-            query = query.replace("'", '"')
-            criteria = json.loads(query)
-
-            for key, value in criteria.items():
-                input_db = self.provider._evaluate_lookup(key, value, False, input_db)
-
-            items = list(input_db.values())
-            result = ResultSet(
-                offset=1, limit=len(items), total=len(items), items=items
-            )
-
-        except json.JSONDecodeError:
-            # FIXME Log Exception
-            raise Exception("Query Malformed")
-
-        if not current_uow:
-            conn.commit()
-            conn.close()
-
-        return result
+    def has_table(self) -> bool:
+        """Always returns True for MemoryProvider as it is always available"""
+        return True
 
 
 operators = {
@@ -502,6 +533,7 @@ operators = {
     "lt": "<",
     "lte": "<=",
     "in": "in",
+    "any": "any",
 }
 
 
@@ -638,5 +670,25 @@ class In(MemoryLookup):
 
     def process_target(self):
         """Ensure target is a list or tuple"""
-        assert type(self.target) in (list, tuple)
-        return super().process_target()
+        target = super().process_target()
+        return f"[{target}]" if not isinstance(target, list) else target
+
+
+@MemoryProvider.register_lookup
+class Any(MemoryLookup):
+    """Any Query for Lists"""
+
+    lookup_name = "any"
+
+    def process_source(self):
+        """Ensure source is a list"""
+        source = super().process_source()
+        return f"[{source}]" if not isinstance(source, list) else source
+
+    def process_target(self):
+        """Ensure target is a list"""
+        target = super().process_target()
+        return f"[{target}]" if not isinstance(target, list) else target
+
+    def as_expression(self):
+        return f"any(x in {self.process_target()} for x in {self.process_source()})"
