@@ -1,12 +1,12 @@
 from abc import abstractmethod
 
-from protean.core.field.basic import Dict, List, String
+from protean.fields import Dict, List, String, Text
 from protean.utils import (
     DomainObjects,
     convert_str_values_to_list,
     derive_element_class,
 )
-from protean.utils.container import BaseContainer
+from protean.utils.container import BaseContainer, OptionsMixin
 
 
 class BaseEmailProvider:
@@ -24,18 +24,11 @@ class BaseEmailProvider:
         self.fail_silently = fail_silently
 
     @abstractmethod
-    def send_email(self, email_messages):
-        """
-        Send one or more EmailMessage objects and return the number of email
-        messages sent.
-        """
-        raise NotImplementedError(
-            "Concrete implementations of BaseEmailBackend "
-            "must override send_email() method"
-        )
+    def send_email(self, email_message):
+        """Send EmailMessage object via registered email provider."""
 
 
-class BaseEmail(BaseContainer):
+class BaseEmail(BaseContainer, OptionsMixin):  # FIXME Remove OptionsMixin
     """Base Email class that should implemented by all Domain Email Messages.
 
     This is also a marker class that is referenced when emails are registered
@@ -44,20 +37,23 @@ class BaseEmail(BaseContainer):
 
     element_type = DomainObjects.EMAIL
 
-    META_OPTIONS = [("provider", "default")]
-
-    def __new__(cls, *args, **kwargs):
-        if cls is BaseEmail:
-            raise TypeError("BaseEmail cannot be instantiated")
-        return super().__new__(cls)
+    @classmethod
+    def _default_options(cls):
+        return [("provider", "default")]
 
     subject = String()
-    data = Dict()
     from_email = String()
     to = List(content_type=String)
     bcc = List(content_type=String)
     cc = List(content_type=String)
     reply_to = String()
+
+    # Supplied content
+    text = Text()
+    html = Text()
+
+    # JSON data with template
+    data = Dict()
     template = String()
 
     def defaults(self):
@@ -83,5 +79,5 @@ class BaseEmail(BaseContainer):
         return [email for email in (self.to + self.cc + self.bcc) if email]
 
 
-def email_factory(element_cls, **kwargs):
+def email_factory(element_cls, domain, **opts):
     return derive_element_class(element_cls, BaseEmail)
