@@ -1,13 +1,22 @@
 """QuerySet Implementation"""
 
+from __future__ import annotations
+
 import copy
 import logging
+from typing import TYPE_CHECKING, Any, Union
 
-from typing import Any, Union
-
+from protean.utils import DomainObjects
+from protean.utils.globals import current_uow
 from protean.utils.query import Q
+from protean.utils.reflection import id_field
 
-logger = logging.getLogger("protean.repository.queryset")
+if TYPE_CHECKING:
+    from protean.core.entity import BaseEntity
+    from protean.domain import Domain
+    from protean.port.dao import BaseDAO
+
+logger = logging.getLogger(__name__)
 
 
 class QuerySet:
@@ -36,18 +45,16 @@ class QuerySet:
 
     def __init__(
         self,
-        owner_dao,
-        domain,
-        entity_cls,
-        criteria=None,
+        owner_dao: BaseDAO,
+        domain: Domain,
+        entity_cls: BaseEntity,
+        criteria: Q = None,
         offset: int = 0,
-        # Aggregates should be loaded in entirety
-        # FIXME Should this limit be removed entirely?
-        limit: int = 1000,
+        limit: int = None,  # No limit by default
         order_by: list = None,
     ):
         """Initialize either with empty preferences (when invoked on an Entity)
-            or carry forward filters and preferences when chained
+        or carry forward filters and preferences when chained
         """
         self._owner_dao = owner_dao
         self._domain = domain
@@ -55,7 +62,9 @@ class QuerySet:
         self._criteria = criteria or Q()
         self._result_cache = None
         self._offset = offset or 0
-        self._limit = limit or 10
+
+        # If an explicit limit is not provided, use the limit from the entity class
+        self._limit = limit or entity_cls.meta_.limit
 
         # `order_by` could be empty, or a string or a list.
         #   Initialize empty list if `order_by` is None
@@ -115,7 +124,8 @@ class QuerySet:
         """Limit number of records"""
         clone = self._clone()
 
-        if isinstance(limit, int):
+        # Assign limit if it is an integer or None
+        if isinstance(limit, int) or limit is None:
             clone._limit = limit
 
         return clone
@@ -139,7 +149,7 @@ class QuerySet:
 
         return clone
 
-    def all(self):
+    def all(self) -> ResultSet:
         """Primary method to fetch data based on filters
 
         Also trigged when the QuerySet is evaluated by calling one of the following methods:
@@ -154,9 +164,6 @@ class QuerySet:
         # Destroy any cached results
         self._result_cache = None
 
-        # Fetch Model class and connected repository from Domain
-        model_cls = self._domain.get_model(self._entity_cls)
-
         # Call the read method of the dao
         results = self._owner_dao._filter(
             self._criteria, self._offset, self._limit, self._order_by
@@ -165,9 +172,30 @@ class QuerySet:
         # Convert the returned results to entity and return it
         entity_items = []
         for item in results.items:
-            entity = model_cls.to_entity(item)
+            entity = self._owner_dao.database_model_cls.to_entity(item)
             entity.state_.mark_retrieved()
+
+            # If we are dealing with an aggregate, we should also update the last event position
+            #   to make use of optimistic concurrency control. This event version will be used
+            #   to check for conflicts when the aggregate is updated.
+            # FIXME: This concurrency control applies only when events are generated. We should
+            #   ensure the same control is applied when aggregates are updated without events.
+            if entity.element_type == DomainObjects.AGGREGATE:
+                # Fetch and sync events version
+                identifier = getattr(entity, id_field(entity).field_name)
+                last_message = self._domain.event_store.store.read_last_message(
+                    f"{entity.meta_.stream_category}-{identifier}"
+                )
+                if last_message:
+                    entity._event_position = last_message.position
+
             entity_items.append(entity)
+
+            # Track aggregate at the UoW level, to be able to perform actions on UoW commit,
+            #   like persisting events raised by the aggregate.
+            if current_uow and entity.element_type == DomainObjects.AGGREGATE:
+                current_uow._add_to_identity_map(entity)
+
         results.items = entity_items
 
         # Cache results
@@ -216,9 +244,6 @@ class QuerySet:
         # Destroy any cached results
         self._result_cache = None
 
-        # Fetch Model class and connected repository from Domain
-        model_cls = self._domain.get_model(self._entity_cls)
-
         try:
             # Call the raw method of the repository
             results = self._owner_dao._raw(query, data)
@@ -226,7 +251,7 @@ class QuerySet:
             # Convert the returned results to entity and return it
             entity_items = []
             for item in results.items:
-                entity = model_cls.to_entity(item)
+                entity = self._owner_dao.database_model_cls.to_entity(item)
                 entity.state_.mark_retrieved()
                 entity_items.append(entity)
             results.items = entity_items
@@ -363,6 +388,11 @@ class QuerySet:
         """Return the first result"""
         return self._data.first
 
+    @property
+    def last(self):
+        """Return the last result"""
+        return self._data.last
+
     @property
     def has_next(self):
         """Return True if there are more values present"""
@@ -372,3 +402,68 @@ class QuerySet:
     def has_prev(self):
         """Return True if there are previous values present"""
         return self._data.has_prev
+
+
+class ResultSet(object):
+    """This is an internal helper class returned by DAO query operations.
+
+    The purpose of this class is to prevent DAO-specific data structures from leaking into the domain layer.
+    It can help check whether results exist, traverse the results, fetch the total number of items and also provide
+    basic pagination support.
+    """
+
+    def __init__(self, offset: int, limit: int, total: int, items: list):
+        # the current offset (zero indexed)
+        self.offset = offset
+        # the number of items to be fetched
+        self.limit = limit
+        # the total number of items matching the query
+        self.total = total
+        # the results
+        self.items = items
+
+    @property
+    def has_prev(self):
+        """Is `True` if the results are a subset of all results"""
+        return bool(self.items) and self.offset > 0
+
+    @property
+    def has_next(self):
+        """Is `True` if more pages exist"""
+        return (self.offset + self.limit) < self.total
+
+    @property
+    def first(self):
+        """Return the first item from results"""
+        if self.items:
+            return self.items[0]
+
+    @property
+    def last(self):
+        """Return the last item from results"""
+        if self.items:
+            return self.items[-1]
+
+    def __bool__(self):
+        """Returns `True` when the resultset is not empty"""
+        return bool(self.items)
+
+    def __iter__(self):
+        """Returns an iterable on items, to support traversal"""
+        return iter(self.items)
+
+    def __len__(self):
+        """Returns number of items in the resultset"""
+        return len(self.items)
+
+    def __repr__(self):
+        return f"<ResultSet: {len(self.items)} items>"
+
+    def to_dict(self):
+        """Return the resultset as a dictionary"""
+        return {
+            "offset": self.offset,
+            "limit": self.limit,
+            "total": self.total,
+            "items": self.items,
+        }
