@@ -1,63 +1,29 @@
+from __future__ import annotations
+
 import logging
-
 from abc import ABCMeta, abstractmethod
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
+from protean.core.database_model import BaseDatabaseModel
 from protean.core.entity import BaseEntity
-from protean.core.field.basic import Auto, Field
-from protean.core.queryset import QuerySet
-from protean.exceptions import ObjectNotFoundError, TooManyObjectsError, ValidationError
-from protean.globals import current_uow
+from protean.core.queryset import QuerySet, ResultSet
+from protean.exceptions import (
+    ExpectedVersionError,
+    ObjectNotFoundError,
+    TooManyObjectsError,
+    ValidationError,
+)
+from protean.fields import Auto, Field
+from protean.port.provider import BaseProvider
+from protean.utils import DomainObjects
+from protean.utils.globals import current_uow
 from protean.utils.query import Q
+from protean.utils.reflection import declared_fields, id_field, unique_fields
 
-logger = logging.getLogger("protean.repository")
+if TYPE_CHECKING:
+    from protean.domain import Domain
 
-
-class ResultSet(object):
-    """This is an internal helper class returned by DAO query operations.
-
-    The purpose of this class is to prevent DAO-specific data structures from leaking into the domain layer.
-    It can help check whether results exist, traverse the results, fetch the total number of items and also provide
-    basic pagination support.
-    """
-
-    def __init__(self, offset: int, limit: int, total: int, items: list):
-        # the current offset (zero indexed)
-        self.offset = offset
-        # the number of items to be displayed on a page.
-        self.limit = limit
-        # the total number of items matching the query
-        self.total = total
-        # the items for the current page
-        self.items = items
-
-    @property
-    def has_prev(self):
-        """Is `True` if the results are a subset of all results"""
-        return bool(self.items) and self.offset > 0
-
-    @property
-    def has_next(self):
-        """Is `True` if more pages exist"""
-        return (self.offset + self.limit) < self.total
-
-    @property
-    def first(self):
-        """Is the first item from results"""
-        if self.items:
-            return self.items[0]
-
-    def __bool__(self):
-        """Returns `True` when the resultset is not empty"""
-        return bool(self.items)
-
-    def __iter__(self):
-        """Returns an iterable on items, to support traversal"""
-        return iter(self.items)
-
-    def __len__(self):
-        """Returns number of items in the resultset"""
-        return len(self.items)
+logger = logging.getLogger(__name__)
 
 
 class BaseDAO(metaclass=ABCMeta):
@@ -74,12 +40,18 @@ class BaseDAO(metaclass=ABCMeta):
     :param domain: the domain of the application this DAO is associated with.
     :param provider: the corresponding provider object of the database implementation, from whom the DAO can
                      request and fetch sessions and connections.
-    :param model_cls: the concrete model class associated with the DAO. The model class is a direct representation
+    :param database_model_cls: the concrete model class associated with the DAO. The model class is a direct representation
                       of an object in ORM/ODM terms or as represented by a python driver.
     :param entity_cls: the domain entity class associated with the DAO.
     """
 
-    def __init__(self, domain, provider, entity_cls, model_cls):
+    def __init__(
+        self,
+        domain: Domain,
+        provider: BaseProvider,
+        entity_cls: BaseEntity,
+        database_model_cls: BaseDatabaseModel,
+    ):
         #: Holds a reference to the domain to which the DAO belongs to.
         self.domain = domain
 
@@ -87,7 +59,7 @@ class BaseDAO(metaclass=ABCMeta):
         self.provider = provider
 
         #: Holds a reference to the model class representation required by the ORM/ODM or the python database driver.
-        self.model_cls = model_cls
+        self.database_model_cls = database_model_cls
 
         #: Holds a reference to the entity class associated with this DAO.
         self.entity_cls = entity_cls
@@ -233,6 +205,15 @@ class BaseDAO(metaclass=ABCMeta):
                      (in case of update statements, for example).
         """
 
+    @abstractmethod
+    def has_table(self) -> bool:
+        """Check if the table/collection exists in the data store.
+
+        Returns True if the table/collection exists, False otherwise.
+
+        :return: Boolean indicating if the table/collection exists
+        """
+
     ######################
     # Life-cycle methods #
     ######################
@@ -256,16 +237,14 @@ class BaseDAO(metaclass=ABCMeta):
 
         # Filter on the ID field of the entity
         filters = {
-            self.entity_cls.meta_.id_field.field_name: identifier,
+            id_field(self.entity_cls).field_name: identifier,
         }
 
         results = self.query.filter(**filters).all()
         if not results:
             raise ObjectNotFoundError(
-                {
-                    "_entity": f"`{self.entity_cls.__name__}` object with identifier {identifier} "
-                    f"does not exist."
-                }
+                f"`{self.entity_cls.__name__}` object with identifier {identifier} "
+                f"does not exist."
             )
 
         if len(results) > 1:
@@ -288,19 +267,15 @@ class BaseDAO(metaclass=ABCMeta):
 
         :param kwargs: named arguments of attribute names and values to filter on.
         """
-        logger.debug(
-            f"Lookup `{self.entity_cls.__name__}` object with values " f"{kwargs}"
-        )
+        logger.debug(f"Lookup `{self.entity_cls.__name__}` object with values {kwargs}")
 
         # Filter for item in the data store
         results = self.query.filter(**kwargs).all()
 
         if not results:
             raise ObjectNotFoundError(
-                {
-                    "_entity": f"`{self.entity_cls.__name__}` object with values {[item for item in kwargs.items()]} "
-                    f"does not exist."
-                }
+                f"`{self.entity_cls.__name__}` object with values {[item for item in kwargs.items()]} "
+                f"does not exist."
             )
 
         if len(results) > 1:
@@ -351,10 +326,10 @@ class BaseDAO(metaclass=ABCMeta):
             self._validate_unique(entity_obj)
 
             # Build the model object and persist into data store
-            model_obj = self._create(self.model_cls.from_entity(entity_obj))
+            model_obj = self._create(self.database_model_cls.from_entity(entity_obj))
 
             # Reverse update auto fields into entity
-            for field_name, field_obj in entity_obj.meta_.declared_fields.items():
+            for field_name, field_obj in declared_fields(entity_obj).items():
                 if isinstance(field_obj, Auto) and not getattr(entity_obj, field_name):
                     if isinstance(model_obj, dict):
                         field_val = model_obj[field_name]
@@ -366,11 +341,33 @@ class BaseDAO(metaclass=ABCMeta):
             # Set Entity status to saved to let everybody know it has been persisted
             entity_obj.state_.mark_saved()
 
+            # Track aggregate at the UoW level, to be able to perform actions on UoW commit,
+            #   like persisting events raised by the aggregate.
+            if current_uow and entity_obj.element_type == DomainObjects.AGGREGATE:
+                current_uow._add_to_identity_map(entity_obj)
+
             return entity_obj
         except ValidationError as exc:
             logger.error(f"Failed creating entity because of {exc}")
             raise
 
+    def _validate_and_update_version(self, entity_obj) -> None:
+        if entity_obj.state_.is_persisted:
+            identifier = getattr(entity_obj, id_field(self.entity_cls).field_name)
+            persisted_entity = self.get(identifier)
+
+            # The version of aggregate in the persistence store should be the same as
+            #   the version we are dealing with.
+            if persisted_entity._version != entity_obj._version:
+                raise ExpectedVersionError(
+                    f"Wrong expected version: {entity_obj._version} "
+                    f"(Aggregate: {self.entity_cls.__name__}({identifier}), Version: {persisted_entity._version})"
+                )
+
+        # Now that we are certain we are dealing with the correct version,
+        #   we can safely update the version to the next version.
+        entity_obj._version = entity_obj._next_version
+
     def save(self, entity_obj):
         """Create or update an entity in the data store, depending on its state. An identity for entity record is
         generated, if not already present.
@@ -387,30 +384,29 @@ class BaseDAO(metaclass=ABCMeta):
         """
         logger.debug(f"Saving `{self.entity_cls.__name__}` object")
 
+        if entity_obj.element_type == DomainObjects.AGGREGATE:
+            self._validate_and_update_version(entity_obj)
+
         try:
             # Build the model object and create it
             if entity_obj.state_.is_persisted:
-                self._update(self.model_cls.from_entity(entity_obj))
+                self._update(self.database_model_cls.from_entity(entity_obj))
             else:
                 # Perform unique checks. Raises validation errors if unique constraints are violated.
                 self._validate_unique(entity_obj)
 
-                # If this is a new entity, generate ID
-                if entity_obj.state_.is_new:
-                    if not getattr(
-                        entity_obj, entity_obj.meta_.id_field.field_name, None
-                    ):
-                        setattr(
-                            entity_obj,
-                            entity_obj.meta_.id_field.field_name,
-                            self.entity_cls.generate_identity(),
-                        )
-
-                self._create(self.model_cls.from_entity(entity_obj))
+                self._create(self.database_model_cls.from_entity(entity_obj))
 
             # Set Entity status to saved to let everybody know it has been persisted
             entity_obj.state_.mark_saved()
 
+            # Track aggregate at the UoW level, to be able to perform actions on UoW commit,
+            #   like persisting events raised by the aggregate.
+            if current_uow and entity_obj.element_type == DomainObjects.AGGREGATE:
+                # The element may have changed from the time it was loaded or may have been
+                #   updated multiple times. We retain the last copy in seen.
+                current_uow._add_to_identity_map(entity_obj)
+
             return entity_obj
         except Exception as exc:
             logger.error(f"Failed saving entity because {exc}")
@@ -445,11 +441,16 @@ class BaseDAO(metaclass=ABCMeta):
             # Do unique checks
             self._validate_unique(entity_obj, create=False)
 
-            self._update(self.model_cls.from_entity(entity_obj))
+            self._update(self.database_model_cls.from_entity(entity_obj))
 
             # Set Entity status to saved to let everybody know it has been persisted
             entity_obj.state_.mark_saved()
 
+            # Track aggregate at the UoW level, to be able to perform actions on UoW commit,
+            #   like persisting events raised by the aggregate.
+            if current_uow and entity_obj.element_type == DomainObjects.AGGREGATE:
+                current_uow._add_to_identity_map(entity_obj)
+
             return entity_obj
         except Exception as exc:
             logger.error(f"Failed updating entity because of {exc}")
@@ -467,7 +468,7 @@ class BaseDAO(metaclass=ABCMeta):
         filters, excludes = {}, {}
 
         # Construct filter criteria based on unique fields defined in Entity class
-        for field_name, field_obj in self.entity_cls.meta_.unique_fields.items():
+        for field_name, field_obj in unique_fields(self.entity_cls).items():
             lookup_value = getattr(entity_obj, field_name, None)
 
             # Ignore empty lookup values
@@ -484,7 +485,7 @@ class BaseDAO(metaclass=ABCMeta):
         # Lookup the objects by filters and raise error if objects exist
         for filter_key, lookup_value in filters.items():
             if self.exists(excludes, **{filter_key: lookup_value}):
-                field_obj = self.entity_cls.meta_.declared_fields[filter_key]
+                field_obj = declared_fields(self.entity_cls)[filter_key]
                 field_obj.fail(
                     "unique",
                     entity_name=self.entity_cls.__name__,
@@ -505,7 +506,7 @@ class BaseDAO(metaclass=ABCMeta):
         """
         try:
             if not entity_obj.state_.is_destroyed:
-                self._delete(self.model_cls.from_entity(entity_obj))
+                self._delete(self.database_model_cls.from_entity(entity_obj))
 
                 # Set Entity status to destroyed to let everybody know the object is no longer referable
                 entity_obj.state_.mark_destroyed()
@@ -577,4 +578,3 @@ class BaseLookup(metaclass=ABCMeta):
 
         Concrete implementation for this method varies from database to database.
         """
-        raise NotImplementedError
