@@ -1,10 +1,11 @@
 """Utility classes and methods for DB Adapters, Repositories and Query Constructors"""
+
 import copy
 import functools
 import inspect
 import logging
 
-logger = logging.getLogger("protean.repository")
+logger = logging.getLogger(__name__)
 
 
 def subclasses(cls):
